/// Josh's User Interface Library
/// A C++20 Library for creating, styling, and rendering of a UI/UX widgets.
/// Developed and Maintained by Josh O'Leary @ Redacted Software.
/// Special Thanks to William Tomasine II and Maxine Hayes.
/// (c) 2024 Redacted Software
/// This work is dedicated to the public domain.

/// @file ImageBase.hpp
/// @desc Base implementation for image drawing that is employed by widgets that wish to draw images.
/// @edit 2024-08-05

#pragma once

#include <Color4.hpp>
#include <J3ML/LinearAlgebra/Vector2.hpp>
#include <JGL/types/Texture.h>

using J3ML::LinearAlgebra::Vector2;

namespace JUI
{
    class ImageBase
    {
    public:
        ImageBase();
        ImageBase(JGL::Texture* texture);
    public:
        [[nodiscard]] JGL::Texture* Content() const { return texture;}
        [[nodiscard]] Color4 Color() const { return image_color;}
        [[nodiscard]] Vector2 Scale() const { return scale;}
        [[nodiscard]] Vector2 Origin() const { return origin;}

        void SetContent(JGL::Texture* content);
        void Color(const Color4& newColor);
        void Scale(const Vector2& newScale);
        void Origin(const Vector2& newOrigin);
    public:
        void Draw(const Vector2& pos, const Vector2& size);
    protected:
        JGL::Texture* texture;
        Color4 image_color = Color4(255,255,255);
        Vector2 scale = Vector2(1,1);
        Vector2 origin = Vector2(0,0);
    private:
    };
}