/// Josh's User Interface Library
/// A C++20 Library for creating, styling, and rendering of a UI/UX widgets.
/// Developed and Maintained by Josh O'Leary @ Redacted Software.
/// Special Thanks to William Tomasine II and Maxine Hayes.
/// (c) 2024 Redacted Software
/// This work is dedicated to the public domain.

/// @file ListLayout.hpp
/// @desc Clickable Button widget.
/// @edit 2024-08-02


#pragma once

#include <JUI/Base/LayoutContainer.hpp>

namespace JUI
{
    class ListLayout : public LayoutContainer
    {
    public:
        ListLayout();
        explicit ListLayout(Widget* parent);
        ~ListLayout() override {}
    };

    namespace LayoutOrder
    {
        enum class H { LEFT, RIGHT };
        enum class V { TOP, BOTTOM };
    }

    /// Lays child elements out in a vertical descending list.
    /// Child element positions are overriden by this widget.
    class VerticalListLayout : public ListLayout
    {
    public:
        VerticalListLayout();
        explicit VerticalListLayout(Widget* parent);

        void ApplyLayout() override
        {
            int consumed_height = 0;
            for (auto& child_widget : children)
            {
                // TODO: Implement widget.LayoutOrder property
                // TODO: Sort children by LayoutOrder
                consumed_height += pad_top.Pixels;
                child_widget->SetPosition({0, consumed_height, 0, 0});
                consumed_height += child_widget->GetAbsoluteSize().y;
                consumed_height += pad_bottom.Pixels;
            }
        }
    };

    class HorizontalListLayout : public ListLayout
    {
    public:
        HorizontalListLayout();
        explicit HorizontalListLayout(Widget* parent);
        void ApplyLayout() override
        {
            if (layout == LayoutOrder::H::LEFT)
            {
                int consumed_width = 0;
                for (auto &child_widget : children)
                {
                    consumed_width += pad_left.Pixels;
                    child_widget->SetPosition({consumed_width, 0, 0, 0});
                    consumed_width += child_widget->GetAbsoluteSize().x;
                    consumed_width += pad_right.Pixels;
                }
            }

            if (layout == LayoutOrder::H::RIGHT)
            {
                int consumed_width = this->GetAbsoluteSize().x;
                for (auto &child_widget : children)
                {
                    consumed_width -= pad_left.Pixels;
                    child_widget->SetPosition({consumed_width, 0, 0, 0});
                    consumed_width -= child_widget->GetAbsoluteSize().x;
                    consumed_width -= pad_right.Pixels;
                }
            }
        }
    protected:
        LayoutOrder::H layout = LayoutOrder::H::LEFT;
    };
}