#include "JGL/JGL.h"
#include <JUI/Widgets/Button.hpp>
#include <jlog/Logger.hpp>

namespace JUI
{
    Button::Button(): Widget() {}

    Button::Button(Widget *parent): Clickable() {
        this->SetParent(parent);
    };

    void Button::Draw() {
        if (!visible)
            return;

        J2D::Begin();
        Vector2 abs_pos = GetAbsolutePosition();
        Vector2 abs_size = GetAbsoluteSize();

        auto root_size = GetFamilyTreeRoot()->GetAbsoluteSize();

        GLint *old_scissor_bounds;
        bool clip_was_enabled;
        // TODO: Re-enable clipping
        if (clips_descendants) {
            clip_was_enabled = glIsEnabled(GL_SCISSOR_TEST);
            if (clip_was_enabled)
                glGetIntegerv(GL_SCISSOR_BOX, old_scissor_bounds);

            float presumed_screen_height = 600;
            glScissor(abs_pos.x, presumed_screen_height-abs_size.y-abs_pos.y, abs_size.x, abs_size.y);
            glEnable(GL_SCISSOR_TEST);
        }

        RectBase::Draw(abs_pos, abs_size);

        // Draw Child Elements with scissor clipping still active
        Widget::Draw();

        // Returns clip to previous state
        if (clips_descendants)
        {
            //glScissor(old_scissor_bounds[0], old_scissor_bounds[1], old_scissor_bounds[2], old_scissor_bounds[3]);

            if (!clip_was_enabled) {}
            glDisable(GL_SCISSOR_TEST);

        }

        J2D::End();
    }


    void Button::OnClick(const Vector2& mouse_pos, const MouseButton& btn)
    {
        Clickable::OnClick(mouse_pos, btn);
        clicked = true;
    }

    void Button::OnRelease(const J3ML::LinearAlgebra::Vector2 &mouse_pos, const JUI::MouseButton &btn,
                           bool still_hovering) {
        Clickable::OnRelease(mouse_pos, btn, still_hovering);
        clicked = false;
    }

    void Button::Update(float delta) {
        hovered = IsMouseInside();

        Hoverable::Update(last_known_mouse_pos, delta);

        // TODO: This is duplicated between here and Window.cpp
        // Will be abstracted away into Clickable shortly.
        if (IsHovered() && mb_state && !prev_mb_state)
        {
            OnClick(last_known_mouse_pos, mbtn);
        }

        if (IsClicked() && !mb_state)
        {
            OnRelease(last_known_mouse_pos, mbtn, IsHovered());
        }



        prev_mb_state = mb_state;
    }

    TextButton::TextButton() : Button(), TextBase() {}
    TextButton::TextButton(Widget* parent) : TextButton() {
        this->SetParent(parent);
    }

    void TextButton::Update(float delta) {
        Button::Update(delta);
        TextBase::Update(delta);
    }

    void TextButton::Draw() {
        Button::Draw();
        TextBase::Draw(this->GetAbsolutePosition(), this->GetAbsoluteSize());
    }
}