#include "JUI/Widgets/Window.hpp"
#include <jlog/Logger.hpp>
#include <cstdlib>


namespace JUI
{
    Window::Window() : Widget(), Clickable(), Hoverable(), RectBase(), Draggable(), Resizable(), Dockable() {
        this->SetPosition({200, 200, 0, 0});
        this->SetSize({400, 200, 0, 0});
        min_size = {400, 200};
        this->BGColor({0,0,0,0});
        this->BorderColor({92,92,192});
        this->SetBorderWidth(2);
        this->SetBorderMode(BorderMode::Outline);

        //this->OnHoverEvent += [&] (Vector2 MousePos) {

        //};

        //this->OnExitEvent += [&] (Vector2 MousePos) {
        //    hovered;
        //};

        /*
        this->OnClickEvent += [&] (Vector2 dummy, MouseButton btn) {
            if (draggable && btn == MouseButton::Left) {
                DEBUG("Window draggable")
                SetDrag(true);
            }

            if (resizable && btn == MouseButton::Right)
                DEBUG("Window resizable")
        };
        */

        // TODO: Move out of Event callback
        this->OnReleaseEvent += [&] (Vector2 dummy, MouseButton btn, bool dummy3) {
            if (dragging) {
                SetDrag(false);
            }
        };

        Viewport = new Rect(this);
        Viewport->SetName("Viewport");
        Viewport->BGColor({64,64,64, 255});
        Viewport->SetSize({0, -20, 1, 1});
        Viewport->SetPosition({0,20,0,0});
        // TODO: Viewport->SetAnchorPoint({0.f, 0.f});
        //Viewport->BorderColor({128, 128, 128, 255});
        //Viewport->SetBorderWidth(1);
        //Viewport->SetPadding(1_px);

        // TODO: Viewport->SetBorderRadius(0);

        Topbar = new Rect(this);
        Topbar->SetPosition({0_px, 0_px});
        Topbar->SetSize({100_percent, 20_px});
        Topbar->BGColor({92,92,192, 255});
        //Topbar->BorderColor({128, 128, 128, 255});
        //Topbar->SetBorderWidth(1);

        TitleLabel = new Text(Topbar);
        TitleLabel->Center();
        TitleLabel->SetContent(title);
        TitleLabel->SetTextSize(titlebar_height);


        // TODO: auto* list = new HorizontalListLayout(Topbar);

        // TODO: exit_btn
        exit_btn = new TextButton(Topbar);
        //exit_btn->SetAnchorPoint({1.f, 0.f});
        //exit_btn->SetSize({30_px, 100_percent});
        exit_btn->SetSize({titlebar_height, titlebar_height, 0, 0});
        //exit_btn->BorderColor({128, 128, 128, 255});
        //exit_btn->SetBaseColor({192, 64, 64, 255});
        //exit_btn->SetHoverColor({255, 0, 0, 255});
        //exit_btn->BGColor({192, 64, 64, 255});
        exit_btn->SetContent("X");
        exit_btn->BGColor({92,92,192, 255});
        //JGL::Font FreeSans
        //exit_btn_text->SetFont(TitleLabel->GetFont());
        exit_btn->SetTextSize(titlebar_height);
        exit_btn->SetTextColor({255, 0, 0});
        //exit_btn->Center();

        exit_btn->OnClickEvent += [&] (auto... _) {
            this->SetVisible(false);
        };

        jlog::Debug(std::format("{} {} {} {}", Topbar->Size().X.Pixels, Topbar->Size().Y.Pixels, Topbar->Size().X.Scale,
                          Topbar->Size().Y.Scale));
        //min_size = exit_btn->GetSize() + Topbar->GetSize() + TitleLabel->Size();
         //= Topbar->GetSize();//+ exit_btn->Size();
        // TODO: fs_btn


    }

    Window::Window(Widget* parent) : Window() {
        this->SetParent(parent);
    }

    Vector2 Window::MinSize() const { return min_size; }

    Vector2 Window::MaxSize() const { return max_size; }

    Vector2 Window::CurrentSize() const { return this->GetAbsoluteSize(); }

    std::string Window::Title() const { return title; }

    bool Window::IsDraggable() const { return draggable; }

    bool Window::IsDockable() const { return dockable; }

    bool Window::IsResizable() const { return resizable; }

    Text *Window::GetTitleInstance() { return TitleLabel; }

    Rect *Window::GetTopbarInstance() { return Topbar; }

    Rect *Window::GetViewportInstance() { return Viewport; }

    void Window::SetDrag(bool d) {
        Draggable::SetDrag(d);
        this->initial_drag_offset = this->last_known_mouse_pos - this->GetAbsolutePosition();
    }

    void Window::SetTitleFont(const Font& f) {
        TitleLabel->SetFont(f);
        exit_btn->SetFont(f);
    }

    void Window::OnClick(const Vector2& mouse_pos, const MouseButton& btn)
    {
        Clickable::OnClick(mouse_pos, btn);
        clicked = true;

        if (draggable && btn == MouseButton::Left)
            this->SetDrag(true);

        if (resizable && btn == MouseButton::Right) {
            this->StartResizing(last_known_mouse_pos);
            this->SetResize(true);
            size_when_restart_began = Size();
        }
    }

    void Window::OnRelease(const J3ML::LinearAlgebra::Vector2 &mouse_pos, const JUI::MouseButton &btn,
                           bool still_hovering) {
        Clickable::OnRelease(mouse_pos, btn, still_hovering);
        clicked = false;
        if (dragging)
            this->SetDrag(false);

        if (resizing)
            this->SetResize(false);
    }

    void Window::Update(float delta) {
        if (dragging) {
            jlog::Debug(std::format("mpos {} {}", last_known_mouse_pos.x, last_known_mouse_pos.y));
            Vector2 mpos = last_known_mouse_pos - initial_drag_offset;
            this->SetPosition(UDim2{(int)mpos.x, (int)mpos.y, 0,0});
        }

        if (resizing) {
            Vector2 mpos = this->last_known_mouse_pos;

            Vector2 resize_amt = mpos - initial_resize_offset;

            SetSize(size_when_restart_began + UDim2(resize_amt.x, resize_amt.y, 0, 0));

            Vector2 abs_size = this->GetAbsoluteSize();

            // Clamp X-axis.
            if (abs_size.x < min_size.x)
            {
                float needed_amt_to_resize = min_size.x - abs_size.x;
                SetSize(Size() + UDim2(needed_amt_to_resize, 0, 0, 0));
            }

            // Clamp Y-axis.
            if (abs_size.y < min_size.y)
            {
                float needed_amt_to_resize = min_size.y - abs_size.y;
                SetSize(Size() + UDim2(0, needed_amt_to_resize, 0, 0));
            }

        }

        Widget::Update(delta);
        hovered = IsMouseInside();

        Hoverable::Update(last_known_mouse_pos, delta);

        // TODO: This is duplicated between here and Window.cpp
        // Will be abstracted away into Clickable shortly.
        if (IsHovered() && mb_state && !prev_mb_state)
        {
            OnClick(last_known_mouse_pos, mbtn);
        }

        if (IsClicked() && !mb_state)
        {
            OnRelease(last_known_mouse_pos, mbtn, IsHovered());
        }

        prev_mb_state = mb_state;

    }
    void Window::Draw() {
        if (!visible) // Duped from Rect.cpp::Draw...
            return;
        RectBase::Draw(this->GetAbsolutePosition(), this->GetAbsoluteSize());
        Widget::Draw();
    }

    void Window::SetTitle(const std::string &title) {
        GetTitleInstance()->SetContent(title);
    }
}