/// Josh's User Interface Library
/// A C++20 Library for creating, styling, and rendering of a UI/UX widgets.
/// Developed and Maintained by Josh O'Leary @ Redacted Software.
/// Special Thanks to William Tomasine II and Maxine Hayes.
/// (c) 2024 Redacted Software
/// This work is dedicated to the public domain.

/// @file Button.hpp
/// @desc Clickable Button widget.
/// @edit 2024-07-31

#pragma once

#include "JUI/Base/Widget.hpp"
#include "JUI/Base/RectBase.hpp"
#include "JUI/Base/TextBase.hpp"

namespace JUI
{

    enum class ButtonState
    {
        Base,
        Hovered,
        Pressed,
        Disabled
    };


    //class Button : public Widget, public RectBase, public Clickable, public Hoverable
    class Button : public Widget, public RectBase, public Clickable, public Hoverable
    {

    public:
        Button();
        explicit Button(Widget* parent);
        ~Button() override {};
    public:
        // TODO: These suffice for now as a proof-of-concept
        // But I want more sophisticated style-animation support
        // for the various states of the button
        Color4 GetHoveredBGColor() const;
        Color4 GetBaseBGColor() const;
        Color4 GetPressedBGColor() const;
        Color4 GetDisabledBGColor() const;

        Color4 GetHoveredBorderColor() const;
        Color4 GetBaseBorderColor() const;
        Color4 GetPressedBorderColor() const;
        Color4 GetDisabledBorderColor() const;

        bool IsEnabled() const;


        void SetEnabled(bool enabled);
        void Draw() override;
        void Update(float delta) override;

        void GotoHoverColor();


        void OnClick(const Vector2& mouse_pos, const MouseButton& btn) override;
        void OnRelease(const Vector2& mouse_pos, const MouseButton& bnt, bool still_hovering) override;
    };

    class TextButton : public Button, public TextBase {
    public:
        TextButton();
        explicit TextButton(Widget* parent);
        ~TextButton() override {};
        void Update(float delta) override;
        void Draw() override;
    };
}